#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import val_in_range
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_helpers import check_internet
import pkg_resources
import unittest
import distro
import requests
import click
import apt


def get_installed_package_versions(find_specific=None):
    packages = {}
    for package in pkg_resources.working_set:
        packages[package.key] = package.version
    if find_specific:
        return packages[find_specific]
    return packages


def get_latest_package_version(package_name):
    response = requests.get(f"https://pypi.org/pypi/{package_name}/json")
    if response.status_code == 200:
        data = response.json()
        return data["info"]["version"]
    else:
        return None


class Test_SIMPLE_software_packages(unittest.TestCase):
    """
    Test USB Devices on Bus
    """
    test = TestBase('test_SIMPLE_software_packages')

    def test_check_internet(self):
        """
        Check if the robot is connected to the internet
        """
        while not check_internet():
            input(click.style("Robot not connected to the Internet. Connect to the Internet and press Enter.",
                              fg='yellow', bold=True))
        self.assertTrue(check_internet(), " Robot not connected to the Internet. Connect the robot")

    def check_if_latest(self, pkg_name):
        ubuntu_to_pip_mapping = {'18.04': 'pip2', '20.04': 'pip3'}
        pip_str = ubuntu_to_pip_mapping[distro.version()]

        installed_version = get_installed_package_versions(pkg_name)
        latest_version = get_latest_package_version(pkg_name)

        print("Found {} : current_version={} | latest_version={}".format(pkg_name, installed_version,
                                                                         latest_version))
        self.test.log_params("latest_{}_version".format(pkg_name), latest_version)
        self.test.log_data("installed_{}_version".format(pkg_name), installed_version)

        self.assertEqual(installed_version, latest_version,
                         msg=f"Newer version v{latest_version} available | run `{pip_str} install -U {pkg_name}`")

    def test_stretch_body(self):
        """
        Test stretch body package version
        """
        self.check_if_latest('hello-robot-stretch-body')

    def test_stretch_body_tools(self):
        """
        Test stretch body tools package version
        """
        self.check_if_latest('hello-robot-stretch-body-tools')

    def test_stretch_factory(self):
        """
        Test stretch factory package version
        """
        self.check_if_latest('hello-robot-stretch-factory')

    def test_stretch_tool_share(self):
        """
        Test stretch tool share package version
        """
        self.check_if_latest('hello-robot-stretch-tool-share')

    def test_realsense_sw_configuration(self):
        """
        Realsense setup correctly
        """
        apt_list = apt.Cache()
        if distro.version() == '18.04':
            self.test.log_data('ros-melodic-librealsense2', 'ros-melodic-librealsense2' in apt_list)
            self.assertTrue('ros-melodic-librealsense2' in apt_list,'ros-melodic-librealsense2 not found in apt')
            self.assertFalse(apt_list['ros-melodic-librealsense2'].is_installed,'Library ros-melodic-librealsense2 is installed')


        if distro.version() == '20.04':
            self.test.log_data('ros-noetic-librealsense2', 'ros-noetic-librealsense2' in apt_list)
            self.assertTrue('ros-noetic-librealsense2' in apt_list,'ros-noetic-librealsense2 not found in apt')

            self.assertFalse(apt_list['ros-noetic-librealsense2'].is_installed,'Library ros-noetic-librealsense2 is installed')

            self.test.log_data('ros-galactic-librealsense2', 'ros-galactic-librealsense2' in apt_list)
            self.assertTrue('ros-galactic-librealsense2' in apt_list,'ros-galactic-librealsense2 not found in apt')

            self.assertFalse(apt_list['ros-galactic-librealsense2'].is_installed,'Library ros-galactic-librealsense2 is installed')


test_suite = TestSuite(test=Test_SIMPLE_software_packages.test, failfast=False)
test_suite.addTest(Test_SIMPLE_software_packages('test_check_internet'))
test_suite.addTest(Test_SIMPLE_software_packages('test_stretch_body'))
test_suite.addTest(Test_SIMPLE_software_packages('test_stretch_body_tools'))
test_suite.addTest(Test_SIMPLE_software_packages('test_stretch_factory'))
test_suite.addTest(Test_SIMPLE_software_packages('test_stretch_tool_share'))
test_suite.addTest(Test_SIMPLE_software_packages('test_realsense_sw_configuration'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import unittest
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_runner import TestRunner
import os
from subprocess import Popen, PIPE


class Test_SIMPLE_capture_system(unittest.TestCase):
    """
    Capture and log stretch system infos
    """

    test = TestBase('test_SIMPLE_capture_system')

    def run_save_log(self, cmd, fn):
        file_path = "{}/{}_{}.log".format(self.test.results_directory_test_specific, fn, self.test.timestamp)
        os.system('{} >> {}'.format(cmd, file_path))

    def test_log_hardware_echo(self):
        """
        Log stretch_hardware_echo.py
        """
        self.run_save_log('stretch_hardware_echo.py', 'stretch_hardware_echo')

    def test_log_usb_topology(self):
        """
        Log lsusb -v -t
        """
        self.run_save_log('lsusb -v -t', 'usb_topology')

    def test_log_stretch_version_script(self):
        """
        log stretch_version.sh
        """
        self.run_save_log('stretch_version.sh', 'stretch_version')

    def test_log_system_info(self):
        """
        Collect system information
        """
        driver_info = {}

        nuc_bios_version = Popen("sudo dmidecode -s bios-version", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                                 close_fds=True).stdout.read().decode().rstrip()
        system_version = Popen("sudo dmidecode -s system-version", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                               close_fds=True).stdout.read().decode().rstrip()
        baseboard_version = Popen("sudo dmidecode -s baseboard-version", shell=True, bufsize=64, stdin=PIPE,
                                  stdout=PIPE, close_fds=True).stdout.read().decode().rstrip()
        processor_version = Popen("sudo dmidecode -s processor-version", shell=True, bufsize=64, stdin=PIPE,
                                  stdout=PIPE, close_fds=True).stdout.read().decode().rstrip()
        kernel_version = Popen("uname -r", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                               close_fds=True).stdout.read().decode().rstrip()

        driver_info['nuc_bios_version'] = nuc_bios_version
        driver_info['system_version'] = system_version
        driver_info['baseboard_version'] = baseboard_version
        driver_info['processor_version'] = processor_version
        driver_info['firmware_vekernel_versionrsion'] = kernel_version

        # check_install_v4l2()
        print("Linux Kernel Version : %s" % (kernel_version))
        print("NUC Bios Version : %s" % (nuc_bios_version))
        print("NUC System Version : %s" % (system_version))
        print("NUC Baseboard Version : %s" % (baseboard_version))
        print("Processor Version : %s" % (processor_version))
        self.test.log_data('system_info', driver_info)


test_suite = TestSuite(test=Test_SIMPLE_capture_system.test, failfast=False)
test_suite.addTest(Test_SIMPLE_capture_system('test_log_hardware_echo'))
test_suite.addTest(Test_SIMPLE_capture_system('test_log_usb_topology'))
test_suite.addTest(Test_SIMPLE_capture_system('test_log_stretch_version_script'))
test_suite.addTest(Test_SIMPLE_capture_system('test_log_system_info'))

if __name__ == '__main__':
    runner = TestRunner(suite=test_suite, doc_verify_fail=False)
    runner.run()



import setuptools
from os import listdir
from os.path import isfile, join

with open("../README.md", "r") as fh:
    long_description = fh.read()

scripts = set()
script_paths = ['./tools']
for script_path in script_paths:
    scripts = scripts.union({script_path + '/' + f for f in listdir(script_path) if isfile(join(script_path, f))})

stretch_diagnostics_tests_dir_list = ['./{}/*'.format(f) for f in listdir('./stretch_diagnostics_tests')]
package_data = {'stretch_diagnostics_tests': stretch_diagnostics_tests_dir_list}

setuptools.setup(
    name="hello_robot_stretch_diagnostics",
    version="0.0.10",
    author="Hello Robot Inc.",
    author_email="support@hello-robot.com",
    description="Stretch Diagnostics",
    long_description=long_description[30:],
    long_description_content_type="text/markdown",
    url="https://github.com/hello-robot/stretch_diagnostics",
    scripts=scripts,
    packages=['stretch_diagnostics', 'stretch_diagnostics_tests'],
    package_data=package_data,
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"
    ],
    install_requires=['hello-robot-stretch-factory>=0.3.12','hello-robot-stretch-body>=0.4.15','stress','xmltodict']
)



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import Scope_Log_Sensor
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.dynamixel_hello_XL430
import stretch_body.dynamixel_XL430
import stretch_body.stretch_gripper
import time
from stretch_body.hello_utils import deg_to_rad,rad_to_deg
import click
class Test_DYNAMIXEL_check_zero_position(unittest.TestCase):
    """
    Test Dynamixel range of motions match expectations
    """
    test = TestBase('test_DYNAMIXEL_check_zero_position')

    def check_zero_position(self,servo,joint,prompt):
        self.assertTrue(servo.do_ping(), msg='Not able to ping servo %s' % joint)
        servo.disable_torque()
        click.secho(prompt, fg="yellow")
        input()
        servo.pull_status()
        bad_zero = abs(servo.status['pos'])>deg_to_rad(10.0)
        print('Measured zero_t of %f for %s. YAML zero_t of %f ticks'%(servo.status['pos_ticks'],joint, servo.params['zero_t']))
        print('Difference of %f degrees' % rad_to_deg(servo.status['pos']))
        msg = 'Measured zero_t of %f for %s is not near YAML zero_t of %f ticks. Check joint for mechanical issues and parameter settings.' % (servo.status['pos_ticks'],joint, servo.params['zero_t'])
        self.test.log_data(joint+'_measured_zero_t', servo.status['pos_ticks'])
        self.test.log_data(joint+'_yaml_zero_t', servo.params['zero_t'])
        self.assertFalse(bad_zero, msg=msg)
        servo.stop()

    def test_check_zero_position_head_tilt(self):
        """
        Check that head_tilt zero is approximately correct
        """
        servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name='head_tilt', chain=None)
        servo.startup()
        self.check_zero_position(servo,'head_tilt','Manually rotate head_tilt so that camera is pointing straight forward\nHit enter when ready.')


    def test_check_zero_position_head_pan(self):
        """
        Check that head_pan zero is approximately correct
        """
        servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name='head_pan', chain=None)
        servo.startup()
        self.check_zero_position(servo,'head_pan','Manually rotate head_pan so that camera is pointing straight forward\nHit enter when ready.')


    def test_check_zero_position_wrist_yaw(self):
        """
        Check that wrist_yaw zero is approximately correct
        """
        servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name='wrist_yaw', chain=None)
        servo.startup()
        if not servo.is_calibrated:
            servo.home()
        self.assertTrue(servo.is_calibrated,msg='Joint wrist_yaw requires calibration. Home joint and try again.')
        self.check_zero_position(servo,'wrist_yaw','Manually rotate wrist yaw so that gripper is pointing straight forward\nHit enter when ready.')


test_suite = TestSuite(test=Test_DYNAMIXEL_check_zero_position.test,failfast=False)

test_suite.addTest(Test_DYNAMIXEL_check_zero_position('test_check_zero_position_head_pan'))
test_suite.addTest(Test_DYNAMIXEL_check_zero_position('test_check_zero_position_head_tilt'))
test_suite.addTest(Test_DYNAMIXEL_check_zero_position('test_check_zero_position_wrist_yaw'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import Scope_Log_Sensor
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.dynamixel_hello_XL430
import stretch_body.dynamixel_XL430
import stretch_body.stretch_gripper
import time
import click

class Test_DYNAMIXEL_measure_range_of_motion(unittest.TestCase):
    """
    Test Dynamixel range of motions match expectations
    """
    test = TestBase('test_DYNAMIXEL_measure_range_of_motion')
    range_t_nominal = {'head_pan':[67.0, 3870.0],'head_tilt':[1726.0, 3347.0],'wrist_yaw':[44.0, 9140.0],'stretch_gripper':[0, 8186]} #TODO: Numbers based on a Stretch 2 (2001), may differ for RE1.

    def measure_range(self,servo,joint):
        self.assertTrue(servo.startup(),msg='Not able to connect to servo %s'%joint)
        self.assertTrue(servo.do_ping(), msg='Not able to ping servo %s' % joint)
        servo.pull_status()

        dr=abs(servo.params['range_t'][0]-servo.params['range_t'][1])
        dn = abs(self.range_t_nominal[joint][0] - self.range_t_nominal[joint][1])
        bad_yaml_range = abs(dr-dn)>0.1*dr #within 10%
        msg='YAML |range_t|=%f for %s is not near expected |range_t| of %f ticks. Try running REx_calibrate_range.py --%s.'%(dn,joint,dr,joint)
        self.assertFalse(bad_yaml_range,msg=msg)

        servo.params['req_calibration'] = 1

        success, measured = servo.home(single_stop=False, move_to_zero=True, delay_at_stop=1.0, save_calibration=False,set_homing_offset=False)
        self.assertTrue(success,msg='Failed to measure joint range')

        dr = abs(measured[0] - measured[1])
        bad_yaml_range = abs(dr-dn)>0.1*dr #within 10%
        msg = 'Measured |range_t|=%f for %s is not near expected |range_t| of %f ticks. Try running REx_calibrate_range.py --%s' % (dn,joint, dr,joint)
        self.assertFalse(bad_yaml_range, msg=msg)

        servo.stop()

        return measured,servo.params['range_t']

    def test_measure_range_head_pan(self):
        """
        Measure range of motion for head_pan
        """
        servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name='head_pan', chain=None)
        measured,yaml=self.measure_range(servo,'head_pan')
        self.test.log_data('measured_range_t', list(measured))
        self.test.log_data('yaml_range_t', list(yaml))

    def test_measure_range_head_tilt(self):
        """
        Measure range of motion for head_tilt
        """
        servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name='head_tilt', chain=None)
        measured,yaml=self.measure_range(servo,'head_tilt')
        self.test.log_data('measured_range_t', list(measured))
        self.test.log_data('yaml_range_t', list(yaml))

    def test_measure_range_wrist_yaw(self):
        """
        Measure range of motion for wrist_yaw
        """
        print()
        click.secho('Ensure wrist yaw is free to move through its range of motion. Hit enter when ready', fg="yellow")
        input()
        servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name='wrist_yaw', chain=None)
        measured,yaml=self.measure_range(servo,'wrist_yaw')
        self.test.log_data('measured_range_t', list(measured))
        self.test.log_data('yaml_range_t', list(yaml))

test_suite = TestSuite(test=Test_DYNAMIXEL_measure_range_of_motion.test,failfast=False)

test_suite.addTest(Test_DYNAMIXEL_measure_range_of_motion('test_measure_range_head_pan'))
test_suite.addTest(Test_DYNAMIXEL_measure_range_of_motion('test_measure_range_head_tilt'))
test_suite.addTest(Test_DYNAMIXEL_measure_range_of_motion('test_measure_range_wrist_yaw'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3

import unittest
import yaml
import os, fnmatch
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_helpers import get_rs_details
import unittest
import subprocess

import sys
from subprocess import Popen, PIPE, STDOUT
from colorama import Fore, Back, Style


class Test_SIMPLE_realsense_status(unittest.TestCase):
    """
    Test USB Devices on Bus
    """
    test = TestBase('test_SIMPLE_realsense_status')

    def test_get_usb_busID(self):
        """
        Gets the USB Bus number and device ID for monitoring
        """
        out = Popen("usb-devices | grep -B 5 -i 'RealSense' | grep -i 'Bus'", shell=True, bufsize=64, stdin=PIPE,stdout=PIPE, close_fds=True).stdout.read().decode()
        self.assertNotEqual(len(out),0,msg='Realsense D435i not found at USB Bus')
        out_list = out.split(' ')
        bus_no = None
        dev_id = None
        for i in range(len(out_list)):
            if out_list[i].find('Bus') != -1:
                bus_no = out_list[i].split('=')[1]
                bus_no = int(bus_no)
            if out_list[i] == 'Dev#=':
                dev_id = int(out_list[i + 2])
        bus_info = {'bus_no': bus_no, 'device_id': dev_id}
        self.test.log_data('usb_bus_info', bus_info)
        print('Realsense D435i found at USB Bus_No : %d | Device ID : %d' % (bus_no, dev_id))

    def test_get_system_info(self):
        """
        Collect realsense driver information
        """
        driver_info = {}

        fw_details = Popen("rs-fw-update -l | grep -i 'firmware'", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,close_fds=True).stdout.read().decode()
        fw_details = fw_details.split(',')[3]
        fw_version = fw_details.split(' ')[-1]
        driver_info['fw_version'] = fw_version

        nuc_bios_version = Popen("sudo dmidecode -s bios-version", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                                 close_fds=True).stdout.read().decode().rstrip()
        system_version = Popen("sudo dmidecode -s system-version", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                               close_fds=True).stdout.read().decode().rstrip()
        baseboard_version = Popen("sudo dmidecode -s baseboard-version", shell=True, bufsize=64, stdin=PIPE,
                                  stdout=PIPE, close_fds=True).stdout.read().decode().rstrip()
        processor_version = Popen("sudo dmidecode -s processor-version", shell=True, bufsize=64, stdin=PIPE,
                                  stdout=PIPE, close_fds=True).stdout.read().decode().rstrip()
        kernel_version = Popen("uname -r", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                               close_fds=True).stdout.read().decode().rstrip()

        driver_info['nuc_bios_version'] = nuc_bios_version
        driver_info['system_version'] = system_version
        driver_info['baseboard_version'] = baseboard_version
        driver_info['processor_version'] = processor_version
        driver_info['firmware_vekernel_versionrsion'] = kernel_version

        # check_install_v4l2()
        print('\nD435i Firmware version: %s\n' % (fw_version))
        print("Linux Kernel Version : %s" % (kernel_version))
        print("NUC Bios Version : %s" % (nuc_bios_version))
        print("NUC System Version : %s" % (system_version))
        print("NUC Baseboard Version : %s" % (baseboard_version))
        print("Processor Version : %s" % (processor_version))
        self.test.log_data('system_info', driver_info)

    def test_USB3_2_connection(self):
        """
        Check that Realsense camera is on USB3.2 connection
        """
        out = Popen("rs-enumerate-devices| grep Usb | grep 3.2", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,close_fds=True).stdout.read()

        if len(out):
            print('Confirmed USB 3.2 connection to realsense device')
        self.assertIsNot(len(out), 0,msg='Did not find USB 3.2 connection to realsense device')

    def test_realsense_on_usb_bus(self):
        """
        Check that Realsense camera is on USB bus.
        """
        print('---- Checking for Intel D435i ----')
        cmd = "lsusb -d 8086:0b3a"
        returned_value = subprocess.call(cmd, shell=True)  # returns the exit code in unix
        self.assertEqual(returned_value,0,'Realsense D435i not found at USB Bus')

    def test_realsense_details(self):
        """
        Capture realsense details and log
        """
        d=get_rs_details()
        self.assertIsNotNone(d,'Not able to launch Realsense driver. It may be conflicting with ROS')
        self.test.log_data('realsense_details',d)

test_suite = TestSuite(test=Test_SIMPLE_realsense_status.test,failfast=False)
test_suite.addTest(Test_SIMPLE_realsense_status('test_realsense_on_usb_bus'))
test_suite.addTest(Test_SIMPLE_realsense_status('test_realsense_details'))
test_suite.addTest(Test_SIMPLE_realsense_status('test_USB3_2_connection'))
test_suite.addTest(Test_SIMPLE_realsense_status('test_get_system_info'))
test_suite.addTest(Test_SIMPLE_realsense_status('test_get_usb_busID'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import Scope_Log_Sensor
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.dynamixel_hello_XL430
import stretch_body.dynamixel_XL430
import stretch_body.stretch_gripper
import time
import click

class Test_DYNAMIXEL_measure_efforts(unittest.TestCase):
    """
    Test Dynamixel efforts to check for mechanical issues
    """
    test = TestBase('test_DYNAMIXEL_measure_efforts')
    effort_limits = {'head_pan':[-5.0,5.0],'head_tilt':[-10.0,20.0],'stretch_gripper':[-45,5],'wrist_yaw':[-10.0,10.0]}

    def measure_effort(self,servo,joint,duration,y_range,lim_lower=None,lim_upper=None):
        self.assertTrue(servo.startup(),msg='Not able to connect to servo %s'%joint)
        self.assertTrue(servo.do_ping(), msg='Not able to ping servo %s' % joint)
        servo.pull_status()
        if servo.params['req_calibration'] and not servo.is_calibrated:
            servo.home()
        self.assertFalse(servo.params['req_calibration'] and not servo.is_calibrated, msg='Joint %srequires calibration. Home joint and try again.' % joint)

        if lim_lower is None:
            lim_lower = min(servo.ticks_to_world_rad(servo.params['range_t'][0]),
                            servo.ticks_to_world_rad(servo.params['range_t'][1]))
        if lim_upper is None:
            lim_upper = max(servo.ticks_to_world_rad(servo.params['range_t'][0]),
                        servo.ticks_to_world_rad(servo.params['range_t'][1]))

        print('Joint limits of: %f to %f'%(lim_lower,lim_upper))
        servo.move_to(lim_lower)
        servo.wait_until_at_setpoint(timeout=6.0)

        #Go pos direction
        servo.pull_status()
        image_fn = self.test.test_result_dir + '/'+'%s_effort_to_upper_limit%s.png' % (joint,self.test.timestamp)
        s=Scope_Log_Sensor(duration=duration, y_range=y_range, title='%s effort to upper limit'%joint, num_points=100, image_fn=image_fn,delay=.01)
        servo.move_to(lim_upper,servo.params['motion']['slow']['vel'],servo.params['motion']['slow']['accel'])
        log={'pos':[],'effort':[]}
        min_eff=0
        max_eff=0
        while s.step(servo.status['effort']):
            log['pos'].append(servo.status['pos'])
            log['effort'].append(servo.status['effort'])
            max_eff=max(log['effort'][-1],max_eff)
            min_eff = min(log['effort'][-1], min_eff)
            #print('Joint: %s. Pos %f. Effort (pct): %f'%(joint,log['pos'][-1],log['effort'][-1]))
            servo.pull_status()


        #Go neg direction
        servo.pull_status()
        image_fn = self.test.test_result_dir + '/'+'%s_effort_to_lower_limit%s.png' % (joint,self.test.timestamp)
        s=Scope_Log_Sensor(duration=duration, y_range=[-50, 50.0], title='%s effort to lower limit'%joint, num_points=100, image_fn=image_fn,delay=.01)
        servo.move_to(lim_lower,servo.params['motion']['slow']['vel'],servo.params['motion']['slow']['accel'])
        log={'pos':[],'effort':[]}
        min_eff=0
        max_eff=0
        while s.step(servo.status['effort']):
            log['pos'].append(servo.status['pos'])
            log['effort'].append(servo.status['effort'])
            max_eff=max(log['effort'][-1],max_eff)
            min_eff = min(log['effort'][-1], min_eff)
            #print('Joint: %s. Pos %f. Effort (pct): %f'%(joint,log['pos'][-1],log['effort'][-1]))
            servo.pull_status()

        eb = (min_eff > self.effort_limits[joint][0]) and (max_eff < self.effort_limits[joint][1])
        msg='Effort of %f|%f for %s required exceeds limits of %f|%f. Check for mechanical obstruction'%(min_eff,max_eff,joint,self.effort_limits[joint][0],self.effort_limits[joint][1])
        self.assertTrue(eb,msg=msg)
        return log

    def test_measure_effort_head_pan(self):
        """
        Measure efforts to move head_pan through joint range
        """
        servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name='head_pan', chain=None)
        data=self.measure_effort(servo,'head_pan',duration=8.0,y_range=[-50, 50.0])
        self.test.log_data('head_pan_effort_to_limits', data)
        servo.stop()

    def test_measure_effort_head_tilt(self):
        """
        Measure efforts to move head_tilt through joint range
        """
        servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name='head_tilt', chain=None)
        data=self.measure_effort(servo,'head_tilt',duration=5.0,y_range=[-50, 50.0])
        self.test.log_data('head_tilt_effort_to_limits', data)
        servo.stop()

    def test_measure_effort_stretch_gripper(self):
        """
        Measure efforts to move stretch_gripper through joint range
        """
        servo = stretch_body.stretch_gripper.StretchGripper()
        data=self.measure_effort(servo,'stretch_gripper',duration=10.0,y_range=[-50, 50.0],lim_lower=servo.poses['open'],lim_upper=servo.poses['close'])
        self.test.log_data('stretch_gripper_effort_to_limits', data)
        servo.stop()

    def test_measure_effort_wrist_yaw(self):
        """
        Measure efforts to move wrist_yaw through joint range
        """
        print()
        click.secho('Ensure wrist yaw is free to move through its range of motion. Hit enter when ready', fg="yellow")
        input()
        servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name='wrist_yaw', chain=None)
        data = self.measure_effort(servo, 'wrist_yaw', duration=7.0, y_range=[-50, 50.0])
        self.test.log_data('wrist_yaw_effort_to_limits', data)
        servo.stop()

test_suite = TestSuite(test=Test_DYNAMIXEL_measure_efforts.test,failfast=False)
test_suite.addTest(Test_DYNAMIXEL_measure_efforts('test_measure_effort_head_pan'))
test_suite.addTest(Test_DYNAMIXEL_measure_efforts('test_measure_effort_head_tilt'))
test_suite.addTest(Test_DYNAMIXEL_measure_efforts('test_measure_effort_stretch_gripper'))
test_suite.addTest(Test_DYNAMIXEL_measure_efforts('test_measure_effort_wrist_yaw'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import Scope_Sensor_vs_Sensor
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_body.arm
import time
import click

class test_ARM_effort_through_range_of_motion(unittest.TestCase):
    """
    Check if arm can move through its range-of-motion using nominal currents
    """
    test = TestBase('test_ARM_effort_through_range_of_motion')

    def test_effort_through_range_of_motion(self):
        """
        Check if arm can move through its range-of-motion using nominal currents
        """
        data = {}
        a = stretch_body.arm.Arm()
        self.assertTrue(a.startup(),'Unable to startup arm')
        #Move through manually set mechanical limits without any safty protections on
        #Log what the worst case motor effort is required
        #Also don't require homing to be done as that may be broken
        a.motor.enable_safety()
        a.motor.disable_sync_mode()
        a.motor.disable_guarded_mode()
        a.push_command()

        #First collect physical hardstop postions
        click.secho('Manually move arm to fully retracted position', fg="yellow")
        click.secho('Hit enter when ready', fg="yellow")
        input()
        a.pull_status()
        data['x_retract_manual']=a.status['pos']
        xr=a.motor.status['pos']

        click.secho('Manually move arm to fully extended position', fg="yellow")
        click.secho('Hit enter when ready', fg="yellow")
        input()
        a.pull_status()
        data['x_extend_manual'] = a.status['pos']
        xe=a.motor.status['pos']

        a.motor.set_motion_limits(xr,xe)
        a.push_command()

        #Check if physical range of motion reasonable
        data['range_of_motion_m']=abs(data['x_extend_manual']-data['x_retract_manual'])
        lim0=a.params['calibration_range_bounds'][0]-.005
        lim1 = a.params['calibration_range_bounds'][1] + .005
        in_bounds = data['range_of_motion_m']>lim0 and data['range_of_motion_m']<lim1
        msg_in_bounds = 'Manual range of motion of %f compared to expected of %f to %f'%(data['range_of_motion_m'],lim0,lim1)
        print(msg_in_bounds)

        eff_min=0
        eff_max=0
        # Move in first direction and collect pos vs effort
        sls = Scope_Sensor_vs_Sensor(yrange=[-100.0, 100], title='Retract Effort (pct) vs Position (m) ')
        #Move to fully retracted (from motor API so don't require homing to be working)
        a.motor.set_command(mode=a.motor.MODE_POS_TRAJ,x_des=xr)
        a.push_command()
        ts=time.time()
        while time.time()-ts<6.0:
            a.pull_status()
            sls.step(a.status['pos']-a.motor_rad_to_translate_m(xr),a.motor.status['effort_pct'])
            eff_min=min(eff_min,a.motor.status['effort_pct'])
            eff_max=max(eff_max,a.motor.status['effort_pct'])
            #print('Pos %f | Effort %f'%(a.status['pos'],a.motor.status['effort_pct']))
            time.sleep(0.01)
        data['log_retract_pos']=sls.data_x
        data['log_retract_effort'] = sls.data_y
        image_fn = self.test.test_result_dir + '/arm_retraction_effort_vs_pos_%s.png' % self.test.timestamp
        sls.savefig(image_fn)

        # Move in second direction and collect pos vs effort
        sls2 = Scope_Sensor_vs_Sensor(yrange=[-100.0, 100], title='Extension Effort (pct) vs Position (m) ')
        # Move to fully extended (from motor API so don't require homing to be working)
        a.motor.set_command(mode=a.motor.MODE_POS_TRAJ, x_des=xe)
        a.push_command()
        ts = time.time()
        while time.time() - ts < 6.0:
            a.pull_status()
            sls2.step(a.status['pos']-a.motor_rad_to_translate_m(xr), a.motor.status['effort_pct'])
            eff_min=min(eff_min,a.motor.status['effort_pct'])
            eff_max=max(eff_max,a.motor.status['effort_pct'])
            #print('Pos %f | Effort %f' % (a.status['pos'], a.motor.status['effort_pct']))
            time.sleep(0.01)
        data['log_extend_pos'] = sls2.data_x
        data['log_extend_effort'] = sls2.data_y
        image_fn = self.test.test_result_dir + '/arm_extension_effort_vs_pos_%s.png' % self.test.timestamp
        sls2.savefig(image_fn)

        data['contact_thresh_max']=a.params['contact_models']['effort_pct']['contact_thresh_max']
        data['contact_thresh_homing'] = a.params['contact_models']['effort_pct']['contact_thresh_homing']
        data['min_effort_pct']=eff_min
        data['max_effort_pct'] = eff_max
        print('Min effort of: %f . Max effort of %f'%(eff_min,eff_max))
        print('Guarded contact efforts of %f to %f'%(a.params['contact_models']['effort_pct']['contact_thresh_max'][0],a.params['contact_models']['effort_pct']['contact_thresh_max'][1]))
        print('Homing guarded contact efforts of %f to %f' % (a.params['contact_models']['effort_pct']['contact_thresh_homing'][0],a.params['contact_models']['effort_pct']['contact_thresh_homing'][1]))
        click.secho('Data collection done.', fg="yellow")
        click.secho('Hit enter to finish', fg="yellow")
        input()

        self.test.log_data('test_effort_through_range_of_motion', data)

        a.stop()
        self.assertTrue(in_bounds, msg_in_bounds)


test_suite = TestSuite(test=test_ARM_effort_through_range_of_motion.test,failfast=False)
test_suite.addTest(test_ARM_effort_through_range_of_motion('test_effort_through_range_of_motion'))


if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import unittest
import os
import shutil
import xacro
import xml.etree.ElementTree as ET
from stretch_body import robot as rb
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_runner import TestRunner


class Test_ROS_urdf(unittest.TestCase):
    """
    Test to check if URDF file conforms with attached end-of-arm tool
    """

    test = TestBase('test_ROS_urdf')

    def get_param_distro_path(self):
        r = rb.Robot()
        r.startup()
        tool = r.params['tool']
        r.stop()

        ros_distro = ['melodic', 'noetic']
        ros2_distro = ['galactic', 'humble']

        distro = os.getenv('ROS_DISTRO')
        home_path = os.getenv('HOME')
        catkin_urdf_path = 'catkin_ws/src/stretch_ros/stretch_description/urdf'
        ament_urdf_path = 'ament_ws/src/stretch_ros2/stretch_description/urdf'

        if distro in ros_distro:
            urdf_path = '{0}/{1}'.format(home_path, catkin_urdf_path)
        elif distro in ros2_distro:
            urdf_path = '{0}/{1}'.format(home_path, ament_urdf_path)
        else:
            self.test.add_hint('Source ROS distro before running test.')

        return tool, distro, urdf_path

    def test_gripper_type(self):
        """
        Test the kind of gripper attached
        """
        
        tool, distro, urdf_path = self.get_param_distro_path()
        tool_type = ['tool_stretch_gripper', 'tool_stretch_dex_wrist', 'tool_dry_erase_holder_v1', 'tool_reactor_wrist', 'tool_usbcam_wrist']
        
        tool_valid = tool in tool_type
        
        if not tool_valid:
            self.test.add_hint('Unidentified gripper type. Add a valid gripper tool to stretch_configuration_params.yaml')

        self.test.log_data('tool_type', tool)
        self.assertTrue(tool_valid)

    def test_endofarm_xacro(self):
        """
        Test if XACRO file corresponds to the end-of-arm tool in params YAML
        """
        
        xacro_tool_sourced = None

        tool, distro, urdf_path = self.get_param_distro_path()
        xacro_path = '{}/stretch_description.xacro'.format(urdf_path)

        xacro_exists = os.path.exists(xacro_path)
        self.assertTrue(xacro_exists)
        if not xacro_exists:
            self.test.add_hint('{} does not exist.'.format(xacro_path))
            return
        
        shutil.copy(xacro_path, '{}/stretch_description.xacro'.format(self.test.results_directory))

        doc = xacro.parse(open(xacro_path))
        robot_description_config = doc.toxml()
        root = ET.fromstring(robot_description_config)

        attrib_list = []
        for child in root:
            try:
                attrib_list.append(child.attrib['filename'])
            except KeyError:
                continue

        tool_dict = {'tool_stretch_gripper': ['stretch_gripper.xacro', 'stretch_gripper_with_puller.xacro'],
                     'tool_stretch_dex_wrist': ['stretch_dex_wrist.xacro'],
                     'tool_dry_erase_holder_v1': ['stretch_dry_erase_marker.xacro'],
                     'tool_reactor_wrist': [],
                     'tool_usbcam_wrist': ['stretch_wrist_USB_board_camera.xacro'],
                     }

        tool_xacro = tool_dict[tool]

        for x in tool_xacro:
            if x in attrib_list:
                xacro_tool_sourced = x

        if not xacro_tool_sourced:
            self.test.add_hint('Add the correct end of arm tool to {}'.format(xacro_path))

        self.test.log_data('xacro_tool_sourced', xacro_tool_sourced)
        self.assertTrue(xacro_tool_sourced)

    def test_endofarm_urdf(self):
        """
        Test if URDF file corresponds to the end-of-arm tool in params YAML
        """

        urdf_configured = False

        tool, distro, urdf_path = self.get_param_distro_path()
        calibrated_urdf_path = '{}/stretch.urdf'.format(urdf_path)

        calibrated_urdf_exists = os.path.exists(calibrated_urdf_path)
        self.assertTrue(calibrated_urdf_exists)
        if not calibrated_urdf_exists:
            self.test.add_hint('{} does not exist.'.format(calibrated_urdf_path))
            return

        shutil.copy(calibrated_urdf_path, '{}/stretch.urdf'.format(self.test.results_directory))

        link_list = [] # Stores list of links in URDF
        tree = ET.parse(calibrated_urdf_path)
        root = tree.getroot()
        for child in root:
            if child.tag == 'link':
                link_list.append(child.attrib['name'])

        # Dictionary of identifying links for tools that confirm the URDF is configured correctly
        id_link = {'tool_stretch_gripper': 'link_gripper',
                   'tool_stretch_dex_wrist': 'link_wrist_pitch',
                   'tool_dry_erase_holder_v1': 'link_dry_erase_holder',
                   'tool_reactor_wrist': '',
                   'tool_usbcam_wrist': 'link_wrist_USB_board_camera',
                   }

        tool_link = id_link[tool]

        if tool_link in link_list:
            urdf_configured = True
        else:
            self.test.add_hint('URDF file stretch.urdf is not configured correctly. Generate URDF using appropriate XACRO file for tool.')
        
        self.test.log_data('urdf_configured', urdf_configured)
        self.assertTrue(urdf_configured)
        

test_suite = TestSuite(test=Test_ROS_urdf.test, failfast=True)

test_suite.addTest(Test_ROS_urdf('test_gripper_type'))
test_suite.addTest(Test_ROS_urdf('test_endofarm_xacro'))
test_suite.addTest(Test_ROS_urdf('test_endofarm_urdf'))

if __name__ == '__main__':
    runner = TestRunner(suite=test_suite, doc_verify_fail=False)
    runner.run()
